//=============================================================================
// VotingReplicationInfo. Coded by .:..: (Ideas from UT2004)
//=============================================================================
class VotingReplicationInfo expands ReplicationInfo
	Config(ClientVoting)
	Transient;

struct MapVotingMapsType
{
	var string MapFileName;
	var int GameTypeIndex,PlayedTimes,MapNoPlayIndex,VotesNow;
	var byte Disabled;
};
struct GameTypeVotesType
{
	var string GameTypeName,ShortName;
};
var MapVotingMapsType MapList[500];
var GameTypeVotesType GameTypeList[10];
var bool bListIsReady,bGameTypesSent,bClientIsAdmin;
var VotingHandler TheVotingHandler;
var int SendingIndex,CurrentlyVotedMap,NumAdminMaps,LastSentInfoX;
var PlayerPawn OldPlayerOwner;
var WindowConsole WindowConsoleOwner;
var bool bOpeningVotingMenu,bAboutToSeeMenu,bAdminMListUpdated,bWantsAdminMenu,bFillingMapList,bFullMapLGiven,
	bMapListUpdated,bGameTypeListUpdated,bVotesUpdated;
var string AdminMapList[1024];
var string LastSentMapN,FirstMapN;
var byte NumGTypes,FillingEntry;
var string GTypeNames[10];
var VotingCmdItem MyItem;

struct CurrentlyVotedMaps
{
	var string MapName,GameName;
	var byte Count;
	var int MapIndex;
};
var CurrentlyVotedMaps VotedMaps[32];

struct MapListInfo
{
	var string MapName;
	var int GameIndex;
};
var MapListInfo FullMaps[500];
var() config bool bDisallowAutoKeyBind;

var() localized string InfoStrText[2],AdminInfoStrText[2],SomeoneString,CastNormalVoteStr,MapVoteWonStr
	,MidGameStr,NotReadyVotedStr,NotReadyWonStr;

replication
{
	// Functions client can call.
	reliable if( Role<ROLE_Authority )
		CastVote,AdminAddMapList,AdminRemoveMapList,ServerFillMapList,AdminSetMapIndex;

	// Functions server can call.
	reliable if( Role==ROLE_Authority )
		BringVotingMenu,ReceiveMap,UpdateVotesCount,ReceiveGame,AllDataReceived,TurnOffVotingMenu
		,GiveClientMaplist,FillGTypeInfo,FillMapInfo,PlayerVotedMap,MapVotingWon,MidGameVBegun;
	
	reliable if( Role==ROLE_Authority && bNetOwner )
		bClientIsAdmin;
}

function PostBeginPlay()
{
	Disable('Tick');
}

simulated function PlayerPawn FindPlayerOwner()
{
	if( OldPlayerOwner!=None )
		Return OldPlayerOwner;
	if( Level.NetMode==NM_DedicatedServer )
		Return None;
	if( PlayerPawn(Owner)!=None )
	{
		OldPlayerOwner = PlayerPawn(Owner);
		Return OldPlayerOwner;
	}
	ForEach AllActors(Class'PlayerPawn',OldPlayerOwner)
	{
		if( OldPlayerOwner.Player!=None && OldPlayerOwner.Player.Console!=None )
			Return OldPlayerOwner;
	}
	OldPlayerOwner = None;
	Return None;
}

simulated function BringDelayedVotingMenu()
{
	bAboutToSeeMenu = True;
	bWantsAdminMenu = False;
	Enable('Tick');
}
simulated function BringDelayedAdminMenu()
{
	bAboutToSeeMenu = True;
	bWantsAdminMenu = True;
	Enable('Tick');
}

// Tell client to shut down voting menu
simulated function TurnOffVotingMenu()
{
	local PlayerPawn P;
	local UWindowWindow U;
	
	if( WindowConsoleOwner==None )
	{
		P = FindPlayerOwner();
		if( P==None )
			Return;
		WindowConsoleOwner = WindowConsole(P.Player.Console);
		if( WindowConsoleOwner==None )
			Return;
	}
	if( WindowConsoleOwner.Root!=None )
	{
		U = WindowConsoleOwner.Root.FindChildWindow(Class'UVotingFrameMenu', True);
		While( U!=None )
		{
			U.Close();
			U = WindowConsoleOwner.Root.FindChildWindow(Class'UVotingFrameMenu', True);
		}
	}
	WindowConsoleOwner.CloseUWindow();
}

// Tell client to open voting menu
simulated function BringVotingMenu( optional bool bShowAdminMenu )
{
	local PlayerPawn P;

	bWantsAdminMenu = bShowAdminMenu;
	if( WindowConsoleOwner==None )
	{
		P = FindPlayerOwner();
		if( P==None )
			Return;
		WindowConsoleOwner = WindowConsole(P.Player.Console);
		if( WindowConsoleOwner==None )
			Return;
	}
	WindowConsoleOwner.bQuickKeyEnable = True;
	WindowConsoleOwner.LaunchUWindow();
	bOpeningVotingMenu = True;
	if( bShowAdminMenu )
		ServerFillMapList();
	Enable('Tick');
}
simulated function OpenUpMenu( optional bool bAdminMenu )
{
	local UVotingFrameMenu UG;

	if( WindowConsoleOwner.Root==None )
		Return;
	if( bAdminMenu )
		UG = UVotingFrameMenu(WindowConsoleOwner.Root.CreateWindow(Class'UVotingAdminMenu',10,10,500,250,,True));
	else UG = UVotingFrameMenu(WindowConsoleOwner.Root.CreateWindow(Class'UVotingFrameMenu',10,10,500,250,,True));
	UG.bUWindowActive = True;
	UG.bTransient = False;
	UG.bLeaveOnscreen = True;
}
// Send client a map info for voting menu
simulated function ReceiveMap( int ID, string MapName, int GIndex, int PlayedTimes, int VotesNow, int NoPlayCount, byte bDis )
{
	MapList[ID].MapFileName = MapName;
	MapList[ID].GameTypeIndex = GIndex;
	MapList[ID].PlayedTimes = PlayedTimes;
	MapList[ID].MapNoPlayIndex = NoPlayCount;
	MapList[ID].VotesNow = VotesNow;
	MapList[ID].Disabled = bDis;
}
// Send to client number of votes change on a map
simulated function UpdateVotesCount( int ID, int NumVotes, int ID2, optional int NumVotes2 )
{
	bVotesUpdated = True;
	MapList[ID].VotesNow = NumVotes;
	if( ID2!=-1 )
		MapList[ID2].VotesNow = NumVotes2;
	if( bListIsReady )
		RefreshVotedMaps();
}
// Send to client a gametype list
simulated function ReceiveGame( byte ID, string GameTypeN, string ShortName )
{
	GameTypeList[ID].GameTypeName = GameTypeN;
	GameTypeList[ID].ShortName = ShortName;
}
// Tell client all data has been received, ok to show voting menu
simulated function AllDataReceived()
{
	local PlayerPawn P;
	local string S,Bind;
	
	bListIsReady = True;
	RefreshVotedMaps();
	P = FindPlayerOwner();
	Bind = FindActiveKey(P,"SHOWVOTINGMENU","X");
	if( bClientIsAdmin )
	{
		if( Bind!="" )
			S = AdminInfoStrText[0];
		else S = AdminInfoStrText[1];
	}
	else if( Bind!="" )
		S = InfoStrText[0];
	else S = InfoStrText[1];
	if( Bind!="" )
		ReplaceText(S,"%k",Bind);
	P.ClientMessage(S,'Pickup');
}
// Client voted a map
function CastVote( int Index )
{
	TheVotingHandler.ClientVoted(Self,PlayerPawn(Owner),Index);
}

// Give full list of all maps that has been voted
simulated function FindVotedMaps( out string MapList[32], out string GTList[32], out int VoteCountList[32], optional out int IDx[32], optional out int Count )
{
	local int i;
	
	For( i=0; i<32; i++ )
	{
		if( VotedMaps[i].MapName=="" )
		{
			Count = i;
			Return;
		}
		MapList[i] = VotedMaps[i].MapName;
		GTList[i] = VotedMaps[i].GameName;
		VoteCountList[i] = VotedMaps[i].Count;
		IDx[i] = VotedMaps[i].MapIndex;
	}
	Count = 32;
}
// Give list of gametypes
simulated function FindAllVoteableGTs( out string GName[10], out string SGName[10], out byte GIndex[10], out byte Count )
{
	local int i;
	
	For( i=0; i<10; i++ )
	{
		if( GameTypeList[i].GameTypeName=="" )
			Continue;
		GName[Count] = GameTypeList[i].GameTypeName;
		SGName[Count] = GameTypeList[i].ShortName;
		GIndex[Count] = i;
		Count++;
	}
}
// Give list of maps to vote for specific gametype
simulated function FindAllVoteableMaps( byte GIDx, out string Maps[500], out int NPLs[500], out int Seq[500], out byte bDi[500], optional out int Idx[500], optional out int Count )
{
	local int i;
	
	For( i=0; i<500; i++ )
	{
		if( MapList[i].MapFileName=="" || MapList[i].GameTypeIndex!=GIDx )
			Continue;
		Maps[Count] = MapList[i].MapFileName;
		NPLs[Count] = MapList[i].PlayedTimes;
		Seq[Count] = MapList[i].MapNoPlayIndex;
		Idx[Count] = i;
		bDi[Count] = MapList[i].Disabled;
		Count++;
	}
}
// Update voted maps list..
simulated function RefreshVotedMaps()
{
	local int i,j;
	
	For( i=0; i<500; i++ )
	{
		if( MapList[i].MapFileName=="" || MapList[i].VotesNow==0 )
			Continue;
		VotedMaps[j].MapName = MapList[i].MapFileName;
		VotedMaps[j].GameName = GameTypeList[MapList[i].GameTypeIndex].GameTypeName;
		VotedMaps[j].Count = MapList[i].VotesNow;
		VotedMaps[j].MapIndex = i;
		j++;
		if( j==32 )
			Break;
	}
	For( i=j; i<32; i++ )
		VotedMaps[i].MapName = "";
}

// Called by VotingHandler whenever its time to start replication
function OpenReplication()
{
	bClientIsAdmin = PlayerPawn(Owner).bAdmin;
	SetTimer(3,False); // Let client connect correctly first.
}

// Message client about voting status
simulated function PlayerVotedMap( PlayerReplicationInfo Other, int MapIndex )
{
	local string PN,S;
	
	if( Other==None )
		PN = SomeoneString;
	else PN = Other.PlayerName;
	if( !bListIsReady )
		S = NotReadyVotedStr;
	else S = CastNormalVoteStr;
	ReplaceText(S,"%n",PN);
	ReplaceText(S,"%m",MapList[MapIndex].MapFileName@"("$GameTypeList[MapList[MapIndex].GameTypeIndex].ShortName$")");
	FindPlayerOwner().ClientMessage(S);
}
// Message mapvoting map won
simulated function MapVotingWon( int MapIndex )
{
	local string S;
	
	if( !bListIsReady )
	{
		FindPlayerOwner().ClientMessage(NotReadyWonStr);
		Return;
	}
	S = MapVoteWonStr;
	ReplaceText(S,"%m",MapList[MapIndex].MapFileName@"("$GameTypeList[MapList[MapIndex].GameTypeIndex].ShortName$")");
	FindPlayerOwner().ClientMessage(S);
}
// Message mid-game voting begun
simulated function MidGameVBegun()
{
	FindPlayerOwner().ClientMessage(MidGameStr);
}

function Timer()
{
	Enable('Tick');
}

simulated function Tick( float Delta )
{
	if( bOpeningVotingMenu )
	{
		bOpeningVotingMenu = False;
		bAboutToSeeMenu = False;
		OpenUpMenu(bWantsAdminMenu);
	}
	else if( bAboutToSeeMenu )
	{
		bAboutToSeeMenu = False;
		BringVotingMenu(bWantsAdminMenu);
		Return;
	}
	if( Level.NetMode==NM_Client )
	{
		Disable('Tick');
		Return;
	}
	if( Owner==None || Owner.bDeleteMe )
	{
		Destroy();
		Return;
	}
	if( bFillingMapList )
	{
		Switch(FillingEntry)
		{
			Case 0:
				LastSentMapN = GetMapName("",LastSentMapN,1);
				if( FirstMapN=="" )
					FirstMapN = LastSentMapN;
				else if( LastSentMapN~=FirstMapN )
				{
					FillingEntry++;
					Return;
				}
				GiveClientMaplist(LastSentMapN);
				Break;
			Case 1:
				SendNextGameType();
				Break;
			Case 2:
				SendNextMList();
				Break;
		}
	}
	if( !bListIsReady )
	{
		if( !bGameTypesSent )
			SendNextGT();
		else SendNextMap();
	}
}

function SendNextMap()
{
	local string S;
	local int i[4];
	local byte by;

	if( Owner==None || Owner.bDeleteMe )
	{
		Destroy();
		Return;
	}
	TheVotingHandler.GetMapInfo(SendingIndex,S,i[0],i[1],i[2],i[3],by);
	if( S=="" )
	{
ItsReady:
		bListIsReady = True;
		AllDataReceived();
		if( (MyItem==None || MyItem.bDeleteMe) )
		{
			MyItem = Spawn(Class'VotingCmdItem',Owner);
			MyItem.ExecutingFor = Self;
			MyItem.GiveTo(Pawn(Owner));
		}
		GoToState('CheckItem');
	}
	else
	{
		ReceiveMap(SendingIndex,S,i[0],i[3],i[1],i[2],by);
		SendingIndex++;
		if( SendingIndex==500 )
		{
			GoTo'ItsReady';
		}
	}
}
function SendNextGT()
{
	local string S[3];
	
	TheVotingHandler.GetGTInfo(SendingIndex,S[0],S[1],S[2]);
	if( S[0]=="" )
	{
		bGameTypesSent = True;
		SendingIndex = 0;
	}
	else
	{
		ReceiveGame(SendingIndex,S[0],S[1]);
		SendingIndex++;
		if( SendingIndex==10 )
		{
			bGameTypesSent = True;
			SendingIndex = 0;
		}
	}
}

function Destroyed()
{
	if( TheVotingHandler!=None )
		TheVotingHandler.ClientDisconnected(Self);
	Super.Destroyed();
}

simulated function GiveClientMaplist( string MapName )
{
	if( NumAdminMaps==ArrayCount(AdminMapList) )
		Return;
	if( Right(MapName,4)~=".unr" )
		MapName = Left(MapName,Len(MapName)-4); // No file extensions!
	AdminMapList[NumAdminMaps] = MapName;
	NumAdminMaps++;
	bAdminMListUpdated = True;
}
simulated function FindAdminMaps( out string AMapList[1024], optional out int Num )
{
	local int i;
	
	For( i=0; i<NumAdminMaps; i++ )
		AMapList[i] = AdminMapList[i];
	Num = NumAdminMaps;
}
function AdminAddMapList( string MapName )
{
	local int i;
	
	if( !bClientIsAdmin || Len(MapName)<=0 )
		Return;
	TheVotingHandler.AddMapToList(Self,MapName);
}
function AdminSetMapIndex( string MapName, int NewIndex )
{
	local int i;
	
	if( !bClientIsAdmin || Len(MapName)<=0 )
		Return;
	TheVotingHandler.SetMapGIndex(Self,MapName,NewIndex);
}
function AdminRemoveMapList( string MapName )
{
	local int i;
	
	if( !bClientIsAdmin || Len(MapName)<=0 )
		Return;
	TheVotingHandler.RemoveMapFromList(Self,MapName);
}

function ServerFillMapList()
{
	if(Playerpawn(Owner) != None)
		bClientIsAdmin=Playerpawn(Owner).bAdmin;
	if( !bClientIsAdmin || bFullMapLGiven )
		Return;
	bFullMapLGiven = True;
	bFillingMapList = True;
	Enable('Tick');
}
simulated function FillGTypeInfo( string GTypeName, string MPrefix, byte Index )
{
	if( Index>NumGTypes )
		NumGTypes = Index;
	if( MPrefix!="" )
		GTypeName = GTypeName@"("$MPrefix$")";
	GTypeNames[Index] = GTypeName;
	bGameTypeListUpdated = True;
}
simulated function FillMapInfo( string MName, byte GIndex, int MNum )
{
	FullMaps[MNum].MapName = MName;
	FullMaps[MNum].GameIndex = GIndex-1;
	bMapListUpdated = True;
}

function SendNextGameType()
{
	local string M,P;
	
	if( LastSentInfoX>=10 )
	{
		FillingEntry++;
		LastSentInfoX = 0;
		Return;
	}
	TheVotingHandler.GetGameTypeInfo(LastSentInfoX,M,P);
	if( Len(M)<=0 )
	{
		FillingEntry++;
		LastSentInfoX = 0;
		Return;
	}
	FillGTypeInfo(M,P,LastSentInfoX);
	LastSentInfoX++;
}
function SendNextMList()
{
	local string M;
	local int G;
	
	if( LastSentInfoX>=500 )
	{
		LastSentInfoX = 0;
		bFillingMapList = False;
		Disable('Tick');
		Return;
	}
	TheVotingHandler.GetMapInfoAdmin(LastSentInfoX,M,G);
	FillMapInfo(M,G+1,LastSentInfoX);
	LastSentInfoX++;
}

simulated function GetActiveMapList( out string MNames[500], out int GIndexs[500] )
{
	local int i;
	
	bMapListUpdated = False;
	For( i=0; i<500; i++ )
	{
		MNames[i] = FullMaps[i].MapName;
		GIndexs[i] = FullMaps[i].GameIndex;
	}
}
simulated function GetActiveGameTypes( out string GNames[10] )
{
	local byte i;
	
	bGameTypeListUpdated = False;
	For( i=0; i<10; i++ )
		GNames[i] = GTypeNames[i];
}

State CheckItem
{
Begin:
	Sleep(1);
	if( Owner==None || Owner.bDeleteMe )
		Destroy();
	if( MyItem==None || MyItem.bDeleteMe || MyItem.Owner!=Owner )
	{
		if( MyItem!=None )
			MyItem.Destroy();
		MyItem = Spawn(Class'VotingCmdItem',Owner);
		MyItem.ExecutingFor = Self;
		MyItem.GiveTo(Pawn(Owner));
	}
	GoTo'Begin';
}


simulated static final function ReplaceText(out string Text, string Replace, string With)
{
	local int i;
	local string Input;

	if ( Text == "" || Replace == "" )
		return;

	Input = Text;
	Text = "";
	i = InStr(Input, Replace);
	while(i != -1)
	{
		Text = Text $ Left(Input, i) $ With;
		Input = Mid(Input, i + Len(Replace));
		i = InStr(Input, Replace);
	}
	Text = Text $ Input;
}

event BroadcastMessage( coerce string Msg, optional bool bBeep, optional name Type );

simulated function string FindActiveKey( PlayerPawn PP, string KeyAlias, string ShouldBeOn )
{
	local byte i;
	local string KeyName, Alias;

	for ( i=0; i<255; i++ )
	{
		KeyName = PP.ConsoleCommand( "KEYNAME "$i );
		if ( KeyName != "" )
		{	
			Alias = PP.ConsoleCommand( "KEYBINDING "$KeyName );
			if( Alias!="" && InStr(Caps(Alias),KeyAlias)!=-1 )
				Return KeyName;
		}
	}
	if( ShouldBeOn!="" && !bDisallowAutoKeyBind )
	{
		Alias = PP.ConsoleCommand("Get Input"@ShouldBeOn);
		if( Alias!="" )
			Alias = Alias$"|";
		PP.ConsoleCommand("Set Input"@ShouldBeOn@Alias$KeyAlias);
		Return ShouldBeOn;
	}
	Return "";
}

defaultproperties
{
				CurrentlyVotedMap=-1
				bDisallowAutoKeyBind=True
				InfoStrText(0)="Press %k key to bring up voting menu"
				InfoStrText(1)="Write 'ShowVotingMenu' to bring up voting menu"
				AdminInfoStrText(0)="%k key for voting menu, 'AdminVMenu' for admin menu"
				AdminInfoStrText(1)="'ShowVotingMenu' for voting menu, 'AdminVMenu' for admin menu"
				SomeoneString="Someone"
				CastNormalVoteStr="%n has voted for %m."
				MapVoteWonStr="%m has won."
				MidGameStr="Mid-Game voting has been inilized"
				NotReadyVotedStr="%n has voted for some map."
				NotReadyWonStr="Some map has won the mapvoting."
				RemoteRole=ROLE_SimulatedProxy
				bAlwaysRelevant=False
}
